
%   this function find all the symmetries shapes in the picture

%   it is given the cluster centers, the edge and labels maps, and two
%   parameters: numTests is in how many points (in pi radians) the algorithm 
%   will test if the shape is symmetric. and Tcount is a treshold of
%   between the points that are considered symmetric and the total tests

function [radiuses, symmetries, centers] = findSymmetries( clustersCenters, edges, labels, numTests, Tcount)

[clustersNum dim] = size(clustersCenters);
[I J] = size(edges);
allAngles = [0 : pi/numTests : pi]; %I check only half the circle (180 degrees)
numAngles = size(allAngles, 2);
radiuses = [];
symmetries = ones(I, J);
centers = [];

for ( p = 1 : clustersNum ) 
    cluster = clustersCenters(p, :);
    clusterX = cluster(2);
    clusterY = cluster(1);
    labelNum = labels(clusterY, clusterX);
    clusterLabel = cluster(3);
    
    tmpRadius = zeros(1, numAngles);
    for ( k = 1 : numAngles )
        angle = allAngles(k);
    
        % find the distance to the edge
        % if the center is on edge
        flag = 1;
        radius = 0;
        while ( flag == 1 )
            newX = clusterX + round(radius*sin(angle));
            newY = clusterY + round(radius*cos(angle));
            if ( newX < 1 || newX > I || newY < 1 || newY > J )
                radius = Inf;
                break;
            end
            if ( edges(newY, newX) == 0 )
                radius = radius + 1;
            else
                flag = 0;
            end
        end % while

        if ( labelNum == clusterLabel )
            flag = 0;
        else
            flag = 1;
        end 
        % if the center is in different cluster
        while( flag == 1)
            newX = clusterX + round(radius*sin(angle));
            newY = clusterY + round(radius*cos(angle));
            if ( newX < 1 || newX > I || newY < 1 || newY > J )
                radius = Inf;
                break;
            end
            if ( clusterLabel == labels(newY, newX) )
                flag = 0;
            end
            radius = radius + 1;
        end % while

        % here I start to check points in the centers' cluster
        if ( radius == Inf ) % reach the end of the pic
            continue;
        end

        flag = 1;
        flagLast = 0;
        lastRadius = Inf;
        while ( flag == 1 )
            newX = clusterX + round(radius*sin(angle));
            newY = clusterY + round(radius*cos(angle));
            if ( newX < 1 || newX > J || newY < 1 || newY > I )
                break;
            end
            if ( labels(newY, newX) == clusterLabel)
                flagLast = 1;
            end
            if ( edges(newY, newX) == 0 && flagLast == 1)
                flagLast = 0;
                lastRadius = radius;
            end
            radius = radius + 1;
        end % while
        if ( lastRadius == Inf )
            continue;
        else
            radius = lastRadius;
        end
        tmpRadius(k) = radius;
    end % for k
    
    % here I have the distance of the first edge
    count = 0;
%     newXY = ones(numTests, 2);
    k = 1;
    for ( angle = 0 : pi/numTests : pi )
        newX = clusterX + round(tmpRadius(k)*sin(angle+pi));
        newY = clusterY + round(tmpRadius(k)*cos(angle+pi));
        k = k + 1;
        if ( edges(newY, newX) == 0 )
            count = count + 1;
%             newXY(count, :) = [newX newY];
        elseif ( newX+1 <= I && edges(newY, newX+1) == 0 )
            count = count + 1;
%             newXY(count, :) = [newX+1 newY];
        elseif ( newX-1 >= 1 && edges(newY, newX-1) == 0 )
            count = count + 1;
%             newXY(count, :) = [newX-1 newY];
        elseif ( newY+1 <= J && edges(newY+1, newX) == 0 )
            count = count + 1;
%             newXY(count, :) = [newX newY+1];
        elseif ( newY-1 >= 1  && edges(newY-1, newX) == 0 )
            count = count + 1;
%             newXY(count, :) = [newX newY-1];
        end
    end % for angle
    
    if ( count / numTests >= Tcount )
        % it is a symmetric form !!!
        radiuses = [radiuses; tmpRadius];
        centers = [centers; clusterX clusterY];
        k = 1;
        for ( angle = 0 : pi/numTests : pi )
            newX = clusterX + round(tmpRadius(k)*sin(angle));
            newY = clusterY + round(tmpRadius(k)*cos(angle));
            symmetries(newY, newX) = 0;
            newX = clusterX + round(tmpRadius(k)*sin(angle+pi));
            newY = clusterY + round(tmpRadius(k)*cos(angle+pi));
            symmetries(newY, newX) = 0;
            k = k + 1;
%         for ( i = 1 : numTests )
%             if ( newXY(i, 1) ~= 0 )
%                 symmetries(newXY(i, 2), newXY(i, 1)) = 0;
%             end
        end % for angle / i
    end %if
end % for p
