
%	the main function of the mean shift algoritm

%   this function also save all its output to a file called 'picFile.mat',
%   for faster run next time (the run function doesn't need to call
%   meanShift function

%	picFile = the file name with the picture.
%   picFormat = the picture format
%	T1 = the relation in kNearest (only there) between the matrix distance
%           and the color distance.
%	T2 = the threshold for ratio between the density around the cluster and 
%           the minimum density alond the lineconecting them.
%	m = number of points in the sample set/
%	h = the radius of a sphere
%	perturbSize = the size of the vector the perturb all the cluster centers
 

function [pic, labels, centers] = meanShift(picFile, picFormat, T1, T2, h, perturbSize, m)
startTime = cputime;

pic = readPic(picFile, picFormat);

[I J] = size(pic);

epsilon = 0.001;
if ( nargin < 6 || m == 0 )
    m = ceil(I/(2*h+1)) * ceil(J/(2*h+1));
end

imshow(pic);
hold on

%%%%%  phase 1
% selection of a sample set 
set = round(sampleSet(pic, m));%, h);

plot(set(:, 2), set(:, 1), '.r');

fprintf('finished phase 1\n');

%%%%%  phase 2
% mean shift procedure applied to the sample set
candidates = zeros(m, 2); 
for (i=1: m)
    candidates(i, :) = meanShiftProcedure(pic, set(i, :), h, epsilon);
end

% plot(candidates(:, 2), candidates(:, 1), '.b');

fprintf('finished phase 2\n');


%%%%% phase 3
% perturb the candidates
newCandidates = perturbCandidates( candidates, perturbSize );
for (i=1: m)
    candidates(i, :) = meanShiftProcedure(pic, newCandidates(i, :), h, epsilon);
end

% plot(candidates(:, 2), candidates(:, 1), 'og');

fprintf('finished phase 3\n');

%%%% phase 4
% derive cluster centers
tempCenters = getClusterCenters( candidates, h );

plot (tempCenters(:, 2), tempCenters(:, 1), 'og');

fprintf('finished phase 4\n');

%%%%  phase 5
% validate the cluster centers
centers = validateCenters( tempCenters, h, pic, T2);
centers = round(centers);

plot (centers(:, 2), centers(:, 1), 'ob');
% hold off

fprintf('finished phase 5\n');

%%%% phase 6
% making the clusters
labels = kNearest( pic, centers, T1);
%scatter(data(:, 2), data(:, 1), 25, clusters);

fprintf('finished phase 6\n');

save( [picFile '.mat'] );

%%%% plot clusters
% plotClusters(labels, centers);%pic, labels, centers);

fprintf('finished mean shift\n');

cpuTime = cputime - startTime